<?php

namespace Drupal\entity_reference_patterns;

use Drupal\Core\Entity\EntityAutocompleteMatcher;
use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\Tags;
use Drupal\entity_reference_patterns\Entity\EntityReferencePatternEntity;

class EntityReferencePatternMatcher extends EntityAutocompleteMatcher {

  /**
   * Gets matched labels based on a given search string.
   */
  public function getMatches($target_type, $selection_handler, $selection_settings, $string = '') {
    $entity_ids = \Drupal::entityQuery('entity_reference_pattern')
      ->condition('type', $target_type)
      ->condition('status', 1)
      ->sort('weight', 'DESC')
      ->range(0, 1)
      ->execute();

    if ($labels = EntityReferencePatternEntity::loadMultiple($entity_ids)) {
      $label_config_entity = reset($labels);
      $criteria = $label_config_entity->getSelectionCriteria();
      $matches = [];

      $options = $selection_settings + [
        'target_type' => $target_type,
        'handler' => $selection_handler,
        'handler_settings' => $selection_settings,
      ];

      $handler = $this->selectionManager->getInstance($options);
      $token = \Drupal::token();

      if (!empty($string)) {
        // Get an array of matching entities.
        $match_operator = !empty($selection_settings['match_operator']) ? $selection_settings['match_operator'] : 'CONTAINS';
        $entity_labels = $handler->getReferenceableEntities($string, $match_operator, 10);

        // Loop through the entities and convert them into autocomplete output.
        foreach ($entity_labels as $values) {
          foreach ($values as $entity_id => $label) {
            $entity = \Drupal::entityTypeManager()->getStorage($target_type)->load($entity_id);
            $entity = \Drupal::service('entity.repository')->getTranslationFromContext($entity);

            if (!empty($criteria['bundles'])) {
              foreach ($criteria as $type => $condition) {
                if ($type == 'bundles') {
                  foreach ($condition as $k => $v) {
                    if ($k == $entity->bundle()) {
                      $label = $token->replace($label_config_entity->getPattern(), [$entity->getEntityType()->id() => $entity], ['clear' => TRUE]);
                    }
                  }
                }
              }
            }
            else {
              $label = $token->replace($label_config_entity->getPattern(), [$entity->getEntityType()->id() => $entity], ['clear' => TRUE]);
            }

            // Just to be safe here we replace ([digital]) petterns so referenced ID could be extracted.
            $label = preg_replace("/\(([\d\h,]+)\)/", "[$1]", $label);

            $key = $label . ' (' . $entity_id . ')';
            // Strip things like starting/trailing white spaces, line breaks and tags.
            $key = preg_replace('/\s\s+/', ' ', str_replace("\n", '', trim(Html::decodeEntities(strip_tags($key)))));
            // Names containing commas or quotes must be wrapped in quotes.
            $key = Tags::encode($key);

            $matches[] = ['value' => $key, 'label' => $label];
          }
        }
      }

      return $matches;
    }
    else {
      return parent::getMatches($target_type, $selection_handler, $selection_settings, $string);
    }
  }

}
