<?php

namespace Drupal\entity_reference_patterns;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\entity_reference_patterns\Entity\EntityReferencePatternEntity;

/**
 * Entity Reference Patterns module helper.
 *
 * @ingroup entity_reference_patterns
 */
class EntityReferencePatterns {

  use StringTranslationTrait;

  /**
   * Implements hook_options_list_alter().
   */
  public function hook_options_list_alter(&$options, $context) {
    /** @var \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository */
    $entity_repository = \Drupal::service('entity.repository');
    $token = \Drupal::token();
    $fieldDefinition = $context['fieldDefinition']->getFieldStorageDefinition();
    $settings = $fieldDefinition->getSettings();
    $new_options = [];
    if (!empty($settings['target_type'])) {
      $target_type = $settings['target_type'];
      foreach ($options as $bundle_label => $items) {
        if ($bundle_label !== '_none' && is_array($items)) {
          foreach ($items as $entity_id => $label) {
            $entity = \Drupal::entityTypeManager()->getStorage($target_type)->load($entity_id);
            // Set the entity in the correct language for display.
            $entity = $entity_repository->getTranslationFromContext($entity);
            $entity_ids = \Drupal::entityQuery('entity_reference_pattern')
              ->condition('type', $entity->getEntityType()->id())
              ->condition('status', 1)
              ->sort('weight', 'DESC')
              ->range(0, 1)
              ->execute();
            if ($labels = EntityReferencePatternEntity::loadMultiple($entity_ids)) {
              $label_config_entity = reset($labels);
              $criteria = $label_config_entity->getSelectionCriteria();

              if (!empty($criteria['bundles'])) {
                foreach ($criteria as $type => $condition) {
                  if ($type == 'bundles') {
                    foreach ($condition as $k => $v) {
                      if ($k == $entity->bundle()) {
                        $new_label = $token->replace($label_config_entity->getPattern(), [$entity->getEntityType()->id() => $entity], ['clear' => TRUE]);
                      }
                    }
                  }
                }
              }
              else {
                $new_label = $token->replace($label_config_entity->getPattern(), [$entity->getEntityType()->id() => $entity], ['clear' => TRUE]);
              }
              $new_options[$bundle_label][$entity_id] = isset($new_label) ? $new_label : $label;
            }
            else {
              $new_options[$bundle_label][$entity_id] = $label;
            }
          }
        }
        else {
          $new_options[$bundle_label] = $items;
        }
      }
    }
    if (!empty($new_options)) {
      $options = $new_options;
    }
  }

}
